<?php
declare(strict_types=1);

/**
 * Model for KTS Feature GO (General Purpose Output) when used as a Relay
 *
 * This KtsFeature model manages an instance of a relay output.
 *
 *  A relay's status can be either inactive or active.
 *  Its configuration consists of what type of notification causes the relay to
 *  become active (major alarm, minor alarm, or nothing/disabled)
 *
 * There are 3 cards that use relay outputs
 *     -Nena
 *     -CTCSS
 *     -Relay Output card
 *
 * @package Model
 * @subpackage Features
 * @access public
 * @author Andrew Mueller
 * @copyright (C) Copyright 2012 Spectracom Corporation, All Rights Reserved
 */

namespace App\Model\Legacy\Fid;

use App\Lib\KtsSource;
use App\Model\Legacy\Alarm;
use App\Model\Legacy\Feature;
use App\Model\Legacy\KtsFeature;
use App\Model\Legacy\Notification;
use App\Model\Legacy\SysFeature;
use Cake\Log\Log;
use Cake\Utility\Hash;

class FidAlarmOutput extends KtsFeature
{
    protected static $schema = [];

    /**
     * Returns the feature ID.
     *
     * @return int
     */
    public static function fid()
    {
        return SysFeature::FID_GO_ALARM;
    }

    /**
     * Returns feature schema.
     * schema() must be called first to be populated;
     *
     * @return array
     */
    protected static function getSchema()
    {
        return self::$schema;
    }

    /**
     * Set the feature schema.
     * schema() must be called first to be populated;
     *
     * @param array $schema schema to set
     * @return void
     */
    protected static function setSchema(array $schema)
    {
        self::$schema = $schema;
    }

    // -----------------------------------------------VIDS--------------------------------------------------------------------------

    //Vids
    public const VID_GO_CA_ALARM = 0x00;

    //----------------------------------------------VID INFORMATION---------------------------------------------------------

    /**
     * Retrieve the schema (set of variable IDs) for this feature
     *
     * @return array containing the schema
     * @see KtsFeature::schema()
     */
    protected static function vids()
    {
        return [
            [
                'id' => self::VID_GO_CA_ALARM,
                'name' => 'alarm',
                'title' => __('Alarm Type'),
                'type' => 'enum',
                'isSetting' => true,
                'isStatus' => true,
                'writable' => false,
                'range' => 'enumAlarm',
            ],
        ];
    }

    /**
     * Returns a shortened status array specifically for the mini status view
     *
     * Overrides the KTSFeature's getMiniStatus
     *
     * @param string $id - feature instance id
     * @return array status
     */
    public static function getMiniStatus(string $id)
    {
        $result = [];

        $fields = ['alarm'];
        $result = self::afterRead(KtsSource::read(
            $id,
            self::schema(),
            $fields,
            KtsSource::FILTER_VIDS_STATUS
        ));

        $result['Alarm'] = Alarm::readRecords();

        // handle read info to correctly display mini-status code
        return $result;
    }

    /**
     * Gets the 'alarm' type from the list of notifications.
     *
     * @param array $results - Results of the read
     * @return array $results - Results of the find after adding the 'alarm' variable
     */
    public static function afterRead($results)
    {
        if (isset($results['id'])) {
            //set a default
            $results['alarm'] = self::ALARM_NONE;

            $feature = Feature::getRecord($results['id']);
            $slot = $feature['Feature']['slot_id'];
            $instance = $feature['Feature']['instance'];

            //get the data to do the mapping
            $notifications = Notification::readRecords();
            //cycle through the notifications and check if the current relay is a minor or major alarm
            foreach ($notifications as $index => $models) {
                if ($models['Notification']['id'] === 25 || $models['Notification']['id'] === 26) {
                    $alarm = self::ALARM_MINOR;
                } elseif ($models['Notification']['id'] === 27 || $models['Notification']['id'] === 28) {
                    $alarm = self::ALARM_MAJOR;
                } elseif ($models['Notification']['id'] === 46) {
                    $alarm = self::ALARM_NONE;
                }
                foreach ($models['Notification']['relays'] as $relay) {
                    if (
                        isset($relay['cardSlot'], $relay['cardInstance'])
                        && $slot === $relay['cardSlot']
                        && $instance === $relay['cardInstance']
                    ) {
                        $results['alarm'] = $alarm;

                        return $results;
                    }
                }
            }
        }

        return $results;
    }

    /**
     * This method intercepts the normal save and instead writes the data to the
     * notcf.conf file.
     *
     * @param array $data data to be updated
     * @return array data to be saved
     */
    public static function beforeUpdate($data)
    {
        if (isset($data['alarm'])) {
            // Read Current value
            $fields = ['alarm'];
            $status = self::afterRead(KtsSource::read(
                $data['id'],
                self::schema(),
                $fields,
                KtsSource::FILTER_VIDS_STATUS
            ));

            //remove the existing values of the relay
            $feature = Feature::getRecord($data['id']);
            $slot = $feature['Feature']['slot_id'];
            $instance = $feature['Feature']['instance'];

            $notifications = Notification::readRecords();
            foreach ($notifications as $index => $models) {
                $id = $models['Notification']['relays'];
                $relays = $models['Notification']['relays'];

                $numRelays = count($relays);
                for ($i = 0; $i < $numRelays; $i++) {
                    if (
                        isset($relays[$i]['cardSlot'], $relays[$i]['cardInstance'])
                        && $slot === $relays[$i]['cardSlot']
                        && $instance === $relays[$i]['cardInstance']
                    ) {
                        unset($notifications[$index]['Notification']['relays'][$i]);
                    }
                }
                $notifications[$index]['Notification']['relays'] =
                    array_values($notifications[$index]['Notification']['relays']);
            }

            $alarmType = intval($data['alarm']);
            $relay = ['cardSlot' => $slot, 'cardInstance' => $instance];

            $currAlarmType = $status['alarm'];

            // Log the change in the journal log
            if ($currAlarmType != $alarmType) {
                $outputId = $instance + 4;
                Log::info(
                    'Alarm relay ' . $instance . ' (slot ' . $slot . ', instance ' . $instance . ', output index '
                        . $outputId . ') changed from ' . self::enumAlarm($currAlarmType) . ' to '
                        . self::enumAlarm($alarmType),
                    'journal'
                );
            }

            //add the new values of the relay
            switch ($alarmType) {
                case 1:
                    $ids = [25, 26];
                    break;
                case 2:
                    $ids = [27, 28];
                    break;
                default:
                    $ids = [46];
                    break;
            }

            foreach ($notifications as $index => $models) {
                if (in_array($models['Notification']['id'], $ids)) {
                    $notifications[$index]['Notification']['relays'][] = $relay;
                }
            }

            // puts in id => [fields => values] format for Notifications update
            $notificationData = Hash::extract($notifications, '{n}.Notification[id>=0]');

            //save the values added
            Notification::updateAll($notificationData);
        }

        return $data;
    }
}
